@@ -231,7 +231,10 @@ def add(args: Optional[List[str]]) -> argparse.ArgumentParser:
     thrown.
 
     """
-    parser = argparse.ArgumentParser(prog=PROG + " add", description=add.__doc__)
+    parser = argparse.ArgumentParser(
+        prog=PROG + " add",
+        description=add.__doc__,
+    )
     parser.add_argument(
         "context",
         nargs="?",
@@ -273,7 +276,8 @@ def replace(args: Optional[List[str]]) -> argparse.ArgumentParser:
 
     """
     parser = argparse.ArgumentParser(
-        prog=PROG + " replace", description=replace.__doc__
+        prog=PROG + " replace",
+        description=replace.__doc__,
     )
     parser.add_argument(
         "context",
@@ -315,14 +319,19 @@ def update(args: Optional[List[str]]) -> argparse.ArgumentParser:
     context.
 
     """
-    parser = argparse.ArgumentParser(prog=PROG + " update", description=update.__doc__)
+    parser = argparse.ArgumentParser(
+        prog=PROG + " update",
+        description=update.__doc__,
+    )
     parser.add_argument(
         "old_context",
         nargs="?",
         help="existing database context, ':' for prompt, or '-' for stdin",
     )
     parser.add_argument(
-        "new_context", nargs="?", help="new database context or ':' for prompt"
+        "new_context",
+        nargs="?",
+        help="new database context or ':' for prompt",
     )
     if args is None:
         return parser
@@ -360,7 +369,11 @@ def dump(args: Optional[List[str]]) -> argparse.ArgumentParser:
 
     """
     parser = argparse.ArgumentParser(prog=PROG + " dump", description=dump.__doc__)
-    parser.add_argument("string", nargs="?", help="substring match for contexts")
+    parser.add_argument(
+        "string",
+        nargs="?",
+        help="substring match for contexts",
+    )
     if args is None:
         return parser
     argsns = parser.parse_args(args)
@@ -377,8 +390,32 @@ def dump(args: Optional[List[str]]) -> argparse.ArgumentParser:
     return parser
 
 
+def get(args: Optional[List[str]]) -> argparse.ArgumentParser:
+    """Print password matching context to stdout."""
+    parser = argparse.ArgumentParser(
+        prog=PROG + " get",
+        description=get.__doc__,
+    )
+    parser.add_argument(
+        "context",
+        nargs="?",
+        help="exact context match, ':' for prompt, or '-' for stdin",
+    )
+    if args is None:
+        return parser
+    argsns = parser.parse_args(args)
+    keyid = get_keyid()
+    db = open_db(keyid)
+    context = retrieve_context(argsns.context, db=db)
+    if context not in db:
+        error(2, "Context '{}' not found".format(context))
+    print(db[context]["password"])
+    return parser
+
+
 def gui(
-    args: Optional[List[str]], method: Optional[str] = os.getenv("IMPASS_XPASTE", None)
+    args: Optional[List[str]],
+    method: Optional[str] = os.getenv("IMPASS_XPASTE", None),
 ) -> argparse.ArgumentParser:
     """Launch minimal GUI.
 
@@ -397,8 +434,15 @@ def gui(
     accessible through the GUI.
 
     """
-    parser = argparse.ArgumentParser(prog=PROG + " gui", description=gui.__doc__)
-    parser.add_argument("string", nargs="?", help="substring match for contexts")
+    parser = argparse.ArgumentParser(
+        prog=PROG + " gui",
+        description=gui.__doc__,
+    )
+    parser.add_argument(
+        "string",
+        nargs="?",
+        help="substring match for contexts",
+    )
     if args is None:
         return parser
     argsns = parser.parse_args(args)
@@ -484,7 +528,10 @@ def remove(args: Optional[List[str]]) -> argparse.ArgumentParser:
     will be thrown.
 
     """
-    parser = argparse.ArgumentParser(prog=PROG + " remove", description=remove.__doc__)
+    parser = argparse.ArgumentParser(
+        prog=PROG + " remove",
+        description=remove.__doc__,
+    )
     parser.add_argument(
         "context",
         nargs="?",
@@ -521,7 +568,8 @@ def remove(args: Optional[List[str]]) -> argparse.ArgumentParser:
 def print_help(args: Optional[List[str]]) -> argparse.ArgumentParser:
     """Full usage or command help (also '-h' after command)."""
     parser = argparse.ArgumentParser(
-        prog=PROG + " help", description=print_help.__doc__
+        prog=PROG + " help",
+        description=print_help.__doc__,
     )
     if args is None:
         return parser
@@ -541,7 +589,8 @@ def print_help(args: Optional[List[str]]) -> argparse.ArgumentParser:
 def version(args: Optional[List[str]]) -> argparse.ArgumentParser:
     """Print version."""
     parser = argparse.ArgumentParser(
-        prog=PROG + " version", description=version.__doc__
+        prog=PROG + " version",
+        description=version.__doc__,
     )
     if args is None:
         return parser
@@ -677,12 +726,14 @@ CMDS: collections.OrderedDict[
         ("replace", replace),
         ("update", update),
         ("dump", dump),
+        ("get", get),
         ("gui", gui),
         ("remove", remove),
         ("help", print_help),
         ("version", version),
     ]
 )
+
 ALIAS = {
     "--version": "version",
     "--help": "help",
