@@ -6,7 +6,7 @@ export SRC_DIRECTORY=$(cd "$TEST_DIRECTORY"/.. && pwd)
 export PYTHONPATH="$SRC_DIRECTORY":"$PYTHONPATH"
 export IMPASS_DB="$TMP_DIRECTORY"/db
 export GNUPGHOME="$TEST_DIRECTORY"/gnupg
-export IMPASS_KEYID=6D3C87EB41EDE1EC8C7CFAFB032FDE87A6EBD73B
+export IMPASS_KEYID=84DCED32C1D6E9DDF52C65D1B2D1C2C1E7EEC6DC
 
 impass() {
     python3 -m impass "$@"
